@@ -14,25 +14,33 @@ __all__ = (
     "FlushByDetailer",
     "RandomFilterDetailer",
     "RandomBandDetailer",
+    "BandSortDetailer",
     "TrackingInfoDetailer",
     "AltAz2RaDecDetailer",
     "StartFieldSequenceDetailer",
     "BandToFilterDetailer",
+    "TagRadialDetailer",
+    "CopyValueDetailer",
+    "LabelRegionDetailer",
+    "LabelDDFDetailer",
+    "LabelRegionsAndDDFs",
 )
 
 import copy
 import warnings
 
+import healpy as hp
 import numpy as np
 
 import rubin_scheduler.scheduler.features as features
-from rubin_scheduler.scheduler.utils import IntRounded, ObservationArray
+from rubin_scheduler.scheduler.utils import CurrentAreaMap, HpInLsstFov, IntRounded, ObservationArray
 from rubin_scheduler.utils import (
     DEFAULT_NSIDE,
     _angular_separation,
     _approx_alt_az2_ra_dec,
     _approx_altaz2pa,
     _approx_ra_dec2_alt_az,
+    ddf_locations,
     pseudo_parallactic_angle,
     rotation_converter,
 )
@@ -59,9 +67,9 @@ class BaseDetailer:
     def add_observations_array(self, observations_array, observations_hpid):
         """Like add_observation, but for loading a whole array of
         observations at a time"""
-
-        for feature in self.survey_features:
-            self.survey_features[feature].add_observations_array(observations_array, observations_hpid)
+        if hasattr(self, "survey_features"):
+            for feature in self.survey_features:
+                self.survey_features[feature].add_observations_array(observations_array, observations_hpid)
 
     def add_observation(self, observation, indx=None):
         """
@@ -73,8 +81,9 @@ class BaseDetailer:
             The indices of the healpix map that the observation overlaps
             with
         """
-        for feature in self.survey_features:
-            self.survey_features[feature].add_observation(observation, indx=indx)
+        if hasattr(self, "survey_features"):
+            for feature in self.survey_features:
+                self.survey_features[feature].add_observation(observation, indx=indx)
 
     def __call__(self, observation_list, conditions):
         """
@@ -93,19 +102,50 @@ class BaseDetailer:
 
 
 class TrackingInfoDetailer(BaseDetailer):
-    """Fill in lots of the different tracking strings for an observation.
-    Does not clobber information that has already been set"""
+    """Add metadata related to target_name, observation_reason and
+    target_name for an observation.
 
-    def __init__(self, target_name=None, science_program=None, observation_reason=None):
+    Does NOT clobber information that has been previously set.
+    Appended to all surveys detailer lists, if not already present,
+    by BaseSurvey using values from survey configuration.
+
+    See https://rtn-096.lsst.io for further information on these observation
+    metadata fields.
+
+    Parameters
+    ----------
+    target_name : `str` or `None`
+        The value to enter into the target_name field of the Observation.
+        With current metadata expectations, this will indicate a specific
+        target name or a general region label. See also LabelRegionDetailer.
+    science_program : `str` or `None`
+        The value to enter into the science_program field of the Observation.
+        With the current configuration of the SchedulerCSC, this corresponds
+        to the JSON BLOCK executed to acquire the visit.
+    observation_reason : `str` or `None`
+        The value to enter into the observation_reason of the Observation.
+        With current metadata expectations, this will indicate something
+        about the observing mode.
+        Spaces in observation_reason will be replaced by underscores,
+        required by camera handling of metadata.
+    """
+
+    def __init__(self, target_name="", science_program="", observation_reason=""):
         self.survey_features = {}
         self.target_name = target_name
         self.science_program = science_program
         self.observation_reason = observation_reason
+        if self.observation_reason is not None:
+            self.observation_reason = self.observation_reason.replace(" ", "_")
         self.keys = ["science_program", "target_name", "observation_reason"]
 
     def __call__(self, observation_array, conditions):
         for key in self.keys:
-            indx = np.where((observation_array[key] == "") | (observation_array[key] is None))[0]
+            indx = np.where(
+                (observation_array[key] == "")
+                | (observation_array[key] is None)
+                | (observation_array[key] == "None")
+            )[0]
             observation_array[key][indx] = getattr(self, key)
 
         return observation_array
@@ -117,13 +157,12 @@ class FlushByDetailer(BaseDetailer):
 
     Parameters
     ----------
-    flush_time : float
+    flush_time : `float`
         The time to flush after the current MJD. Default 60 minutes
     """
 
-    def __init__(self, flush_time=60, nside=DEFAULT_NSIDE):
+    def __init__(self, flush_time=60):
         self.survey_features = {}
-        self.nside = nside
         self.flush_time = flush_time / 60 / 24.0
 
     def __call__(self, observation_array, conditions):
@@ -231,6 +270,57 @@ class RandomBandDetailer(BaseDetailer):
         return observation_array
 
 
+class BandSortDetailer(BaseDetailer):
+    """Detailer that sorts an array of observations by specified
+    band order, in order to minimize filter changes.
+
+    Useful for scripted surveys with many filter changes like DDFs.
+
+    Parameters
+    ----------
+    desired_band_order : `str`
+        The desired band order. Default of None retains order
+        and only moves observations up if they match the current filter.
+    loaded_first : `bool`
+        If True, then the currently-in-use filter is always moved to
+        the start of the desired band order, to remove the first filter change.
+    """
+
+    def __init__(
+        self,
+        desired_band_order: str = None,
+        loaded_first: bool = True,
+    ):
+        self.desired_band_order = desired_band_order
+        self.loaded_first = loaded_first
+
+    def __call__(
+        self, observation_array: ObservationArray, conditions: features.Conditions
+    ) -> ObservationArray:
+
+        order_to_set = copy.copy(self.desired_band_order)
+
+        u_vals, indx = np.unique(observation_array["band"], return_index=True)
+        # If we have only 1 filter, nothing to sort, so return
+        if np.size(u_vals) == 1:
+            return observation_array
+
+        if order_to_set is None:
+            # unique values, in order they appear, as a string
+            order_to_set = "".join(u_vals[np.argsort(indx)].tolist())
+
+        if self.loaded_first:
+            order_to_set = order_to_set.replace(conditions.current_band, "")
+            order_to_set = conditions.current_band + order_to_set
+        indicies = []
+        for bandname in order_to_set:
+            indicies.append(np.where(observation_array["band"] == bandname)[0])
+
+        indices = np.concatenate(indicies)
+
+        return observation_array[indices]
+
+
 class ParallacticRotationDetailer(BaseDetailer):
     """Set the rotator to near the parallactic angle"""
 
@@ -468,8 +558,7 @@ class FixedSkyAngleDetailer(BaseDetailer):
 
 
 class CloseAltDetailer(BaseDetailer):
-    """
-    re-order a list of observations so that the closest in altitude to
+    """Re-order a list of observations so that the closest in altitude to
     the current pointing is first.
 
     Parameters
@@ -632,3 +721,256 @@ class TwilightTripleDetailer(BaseDetailer):
                 sub_arr["scheduler_note"] = np.char.add(sub_arr["scheduler_note"], ", %i" % i)
             out_obs.append(sub_arr)
         return np.concatenate(out_obs)
+
+
+class TagRadialDetailer(BaseDetailer):
+    """Update target_name and scheduler note based on visit location.
+
+    Parameters
+    ----------
+    radius : `float`
+        How close to a location on the sky a visit must be to
+        get tagged. Defualr 5.6 (degrees).
+    ra : `float`
+        RA of the location. Default 0 (degrees).
+    dec : `float`
+        Dec of the location. Default -30 (degrees).
+    note_append : `str`
+        The string to append to the visit scheduler_note.
+        Default ", deep area".
+    target_prefix : `str`
+        String to add to the start of the target_name field.
+        Default "deep "
+    target_name : `str`
+        Target name to use. Overrides name generater with target_prefix.
+        Default None.
+    """
+
+    def __init__(
+        self, radius=5.6, ra=0, dec=-30.0, note_append=", deep area", target_prefix="deep ", target_name=None
+    ):
+        self.survey_features = {}
+        self.radius = np.radians(radius)
+        self.ra = np.radians(ra)
+        self.dec = np.radians(dec)
+        self.note_append = note_append
+        self.target_name = target_prefix + f"{np.degrees(self.ra):.3f} {np.degrees(self.dec):.3f}"
+        if target_name is not None:
+            self.target_name = target_name
+
+    def __call__(self, obsarray, conditions):
+        distances = _angular_separation(obsarray["RA"], obsarray["dec"], self.ra, self.dec)
+        if np.size(distances) == 1:
+            distances = np.array([distances])
+        in_region = np.where(distances <= self.radius)[0]
+        obsarray["target_name"][in_region] = self.target_name
+        obsarray["scheduler_note"][in_region] = np.char.add(
+            obsarray["scheduler_note"][in_region], self.note_append
+        )
+        return obsarray
+
+
+class LabelRegionDetailer(BaseDetailer):
+    """Label which region(s) of the footprint we are in.
+
+    Parameters
+    ----------
+    label_array : `np.array`
+        A HEALpix array of strings that are labels for each HEALpix.
+        Default of None uses CurrentAreaMap to load labels.
+    field_for_label : `str`
+        Which ObservationArray field should be modified.
+        Default "target_name".
+    camera : `str`
+        Which camera model to use to convert a pointing to
+        HEALpix IDs. Default "LSST".
+    append : `bool`
+        Should the labels be appended to `field_for_label`
+        (True, default), or clobber (False).
+    separator : `str`
+        If append is True, what string to use when appending.
+        Default ", ".
+    """
+
+    def __init__(
+        self, label_array=None, field_for_label="target_name", camera="LSST", append=True, separator=", "
+    ):
+        if label_array is None:
+            sky = CurrentAreaMap()
+            _footprints, self.label_array = sky.return_maps()
+        else:
+            self.label_array = label_array
+        self.field_for_label = field_for_label
+        nside = hp.npix2nside(np.size(self.label_array))
+        self.append = append
+        self.separator = separator
+
+        self.remove_vals = set(["", "", "None"])
+
+        if camera == "LSST":
+            self.pointing2hpindx = HpInLsstFov(nside=nside)
+        else:
+            raise ValueError("Unknown camera")
+
+    def __call__(self, obs_array, conditions):
+        for i in np.arange(obs_array.size):
+            indx = self.pointing2hpindx(obs_array["RA"][i], obs_array["dec"][i])
+            labels = np.unique(self.label_array[indx])
+            # ignore things that are out of bounds
+            labels = labels[np.where(labels != "")]
+            # If there are no applicable labels, identify as OutOfFootprint
+            if np.size(labels) == 0:
+                result = set(["OOF"])
+            else:
+                result = set(labels)
+
+            if self.append:
+                prev_values = set(obs_array[self.field_for_label][i].split(self.separator))
+                new_values = prev_values.union(result).difference(self.remove_vals)
+            else:
+                new_values = result.difference(self.remove_vals)
+            obs_array[self.field_for_label][i] = self.separator.join(new_values)
+        return obs_array
+
+
+class LabelDDFDetailer(BaseDetailer):
+    """Label if an observation is close enough to a
+    DDF location to be tagged.
+
+    Parameters
+    ----------
+    ddf_locations : `dict`
+        Dictionary with keys of DDF names and values of RA,dec
+        pairs (in degrees). Default of None uses ddf_locations
+        utility to get default locations and names.
+    field_for_label : `str`
+        Which ObservationArray field should be modified.
+        Default "target_name".
+    append : `bool`
+        Should the labels be appended to `field_for_label`
+        (True, default), or clobber (False).
+    match_radius : `float`
+        The radius away an observation can be from a DDF center
+        and still be tagged. Default 2.0 (degrees)
+    separator : `str`
+        If append is True, what string to use when appending.
+        Default ", ".
+    """
+
+    def __init__(
+        self, ddf_location=None, field_for_label="target_name", append=True, match_radius=2.0, separator=", "
+    ):
+        self.field_for_label = field_for_label
+        self.append = append
+        self.match_radius = np.radians(match_radius)
+        self.separator = separator
+        if ddf_location is None:
+            self.ddf_locations = ddf_locations()
+        else:
+            self.ddf_locations = ddf_locations
+
+        for key in self.ddf_locations:
+            self.ddf_locations[key] = np.radians(self.ddf_locations[key])
+
+        self.remove_vals = set(["", "", "None"])
+
+    def __call__(self, obs_array, conditions):
+
+        for name in self.ddf_locations:
+            distances = _angular_separation(
+                self.ddf_locations[name][0], self.ddf_locations[name][1], obs_array["RA"], obs_array["dec"]
+            )
+            distances = np.atleast_1d(distances)
+            # Indexes of observations in obs_array which overlap the DDF
+            indxes = np.where(distances <= self.match_radius)[0]
+
+            name_set = set([f"DDF {name}"])
+            if indxes.size > 0:
+                for indx in indxes:
+                    if self.append:
+                        prev_values = set(obs_array[self.field_for_label][indx].split(self.separator))
+                        new_values = prev_values.union(name_set).difference(self.remove_vals)
+                    else:
+                        new_values = name_set.difference(self.remove_vals)
+                    obs_array[self.field_for_label][indx] = self.separator.join(new_values)
+        return obs_array
+
+
+class LabelRegionsAndDDFs(BaseDetailer):
+    """Run both LabelRegionDetailer and LabelDDFDetailer.
+
+    Will always append onto current content of target_name.
+
+    Parameters
+    ----------
+    label_array : `np.array`
+        A HEALpix array of strings that are labels for each HEALpix.
+        Default of None uses CurrentAreaMap to load labels.
+    field_for_label : `str`
+        Which ObservationArray field should be modified.
+        Default "target_name".
+    camera : `str`
+        Which camera model to use to convert a pointing to
+        HEALpix IDs. Default "LSST".
+    ddf_locations : `dict`
+        Dictionary with keys of DDF names and values of RA,dec
+        pairs (in degrees). Default of None uses ddf_locations
+        utility to get default locations and names.
+    match_radius : `float`
+        The radius away an observation can be from a DDF center
+        and still be tagged. Default 2.0 (degrees)
+    separator : `str`
+        If append is True, what string to use when appending.
+        Default ", ".
+    """
+
+    def __init__(
+        self,
+        label_array=None,
+        field_for_label="target_name",
+        camera="LSST",
+        ddf_location=None,
+        match_radius=2.0,
+        separator=", ",
+    ):
+        self.label_detailer = LabelRegionDetailer(
+            label_array=label_array,
+            field_for_label=field_for_label,
+            camera=camera,
+            append=True,
+            separator=separator,
+        )
+        self.ddf_detailer = LabelDDFDetailer(
+            ddf_location=ddf_location,
+            field_for_label=field_for_label,
+            append=True,
+            match_radius=match_radius,
+            separator=separator,
+        )
+
+    def __call__(self, obs_array, conditions):
+
+        result = self.label_detailer(obs_array, conditions)
+        result = self.ddf_detailer(result, conditions)
+        return result
+
+
+class CopyValueDetailer(BaseDetailer):
+    """Copy a value from one observation array column to another
+
+    Parameters
+    ----------
+    source : `str`
+        The name of the source column.
+    destination : `str`
+        Name of the destination column.
+    """
+
+    def __init__(self, source, destination):
+        super().__init__()
+        self.source = source
+        self.destination = destination
+
+    def __call__(self, obs_array, conditions):
+        obs_array[self.destination] = obs_array[self.source]
+        return obs_array
