@@ -10,7 +10,6 @@ import pandas as pd
 
 from rubin_scheduler.scheduler.detailers import TrackingInfoDetailer, ZeroRotDetailer
 from rubin_scheduler.scheduler.utils import (
-    HpInLsstFov,
     ObservationArray,
     comcam_tessellate,
     rotx,
@@ -125,13 +124,29 @@ class BaseSurvey:
 
         # Strings to pass onto observations
         if (target_name is not None) | (science_program is not None) | (observation_reason is not None):
-            self.detailers.append(
-                TrackingInfoDetailer(
-                    target_name=target_name,
-                    science_program=science_program,
-                    observation_reason=observation_reason,
+            should_have_tracking_detailer = True
+        else:
+            should_have_tracking_detailer = False
+        if should_have_tracking_detailer:
+            # Check if one already present - will use that if so.
+            has_tracking_detailer = False
+            for detailer in self.detailers:
+                if isinstance(detailer, TrackingInfoDetailer):
+                    has_tracking_detailer = True
+                    break
+            if has_tracking_detailer:
+                warnings.warn(
+                    f"Survey {self.survey_name} has a tracking detailer but target_name,"
+                    f"observation_reason or science_program also set (and will ignore)."
+                )
+            else:
+                self.detailers.append(
+                    TrackingInfoDetailer(
+                        target_name=target_name,
+                        science_program=science_program,
+                        observation_reason=observation_reason,
+                    )
                 )
-            )
 
     @cached_property
     def roi_hpid(self):
@@ -147,6 +162,14 @@ class BaseSurvey:
 
         return mask
 
+    def update_conditions(self, conditions):
+        """Pass the latest conditions to the survey.
+
+        Usually not relevant, but ScriptedSurveys and ToOs
+        may want to update.
+        """
+        pass
+
     def _generate_survey_name(self):
         self.survey_name = ""
 
@@ -489,7 +512,7 @@ class BaseMarkovSurvey(BaseSurvey):
         npositions=7305,
         target_name=None,
         science_program=None,
-        observation_reason=None,
+        observation_reason="Area",
     ):
         super(BaseMarkovSurvey, self).__init__(
             basis_functions=basis_functions,
@@ -525,7 +548,6 @@ class BaseMarkovSurvey(BaseSurvey):
         else:
             self.fields_init = fields
         self.fields = self.fields_init.copy()
-        self.hp2fields = np.array([])
         self._hp2fieldsetup(self.fields["RA"], self.fields["dec"])
 
         if smoothing_kernel is not None:
@@ -542,7 +564,13 @@ class BaseMarkovSurvey(BaseSurvey):
         self.night = -1
 
         if dither in [True, False]:
-            warnings.warn("setting dither to bool deprecated, swapping to dither='night'", FutureWarning)
+            survey_name = self.survey_name
+            if len(survey_name) == 0:
+                survey_name = self.__class__.__name__
+            warnings.warn(
+                f"setting dither to bool deprecated, swapping to dither='night' ({survey_name})",
+                FutureWarning,
+            )
             dither = "night"
         if dither not in ["night", "call", None]:
             raise ValueError("dither kwarg must be one of 'night', 'call', or None.")
@@ -597,19 +625,15 @@ class BaseMarkovSurvey(BaseSurvey):
         dec : `float`
             The decs of the possible pointings (radians)
         """
-        self.hp2fields = np.zeros(hp.nside2npix(self.nside), dtype=int)
-        if self.camera == "LSST":
-            pointing2hpindx = HpInLsstFov(nside=self.nside)
-            for i in range(len(ra)):
-                hpindx = pointing2hpindx(ra[i], dec[i], rotSkyPos=0.0)
-                self.hp2fields[hpindx] = i
-        elif self.camera == "comcam":
-            # Let's just map each healpix to the closest field location
-            tree = _build_tree(ra, dec)
-            hp_ra, hp_dec = _hpid2_ra_dec(self.nside, np.arange(hp.nside2npix(self.nside)))
-            x, y, z = _xyz_from_ra_dec(hp_ra, hp_dec)
-            dist, ind = tree.query(np.vstack([x, y, z]).T, k=1)
-            self.hp2fields = ind
+
+        # Let's just map each healpix to the closest field location
+        tree = _build_tree(ra, dec)
+        hp_ra, hp_dec = _hpid2_ra_dec(self.nside, np.arange(hp.nside2npix(self.nside)))
+        x, y, z = _xyz_from_ra_dec(hp_ra, hp_dec)
+        dist, ind = tree.query(np.vstack([x, y, z]).T, k=1)
+
+        # XXX--maybe add a check that distance isn't too large
+        self.hp2fields = ind
 
     def _spin_fields(self, indx, lon=None, lat=None, lon2=None):
         """Spin the field tessellation to generate a random orientation
