@@ -1,10 +1,13 @@
 __all__ = ("ToOScriptedSurvey", "gen_too_surveys")
 
 import warnings
-from copy import copy
+from copy import copy, deepcopy
 
+import healpy as hp
 import numpy as np
 
+import rubin_scheduler.scheduler.basis_functions as basis_functions
+from rubin_scheduler.scheduler.detailers import BandPickToODetailer, TrackingInfoDetailer
 from rubin_scheduler.scheduler.surveys import BaseMarkovSurvey, ScriptedSurvey
 from rubin_scheduler.scheduler.utils import (
     ScheduledObservationArray,
@@ -14,7 +17,13 @@ from rubin_scheduler.scheduler.utils import (
     xyz2thetaphi,
 )
 from rubin_scheduler.site_models import _read_fields
-from rubin_scheduler.utils import DEFAULT_NSIDE, _approx_ra_dec2_alt_az, _ra_dec2_hpid
+from rubin_scheduler.utils import (
+    DEFAULT_NSIDE,
+    _approx_ra_dec2_alt_az,
+    _build_tree,
+    _hpid2_ra_dec,
+    _ra_dec2_hpid,
+)
 
 
 def rotx(theta, x, y, z):
@@ -66,6 +75,17 @@ class ToOScriptedSurvey(ScriptedSurvey, BaseMarkovSurvey):
     split_long_div : `float`
         Time to divide the exposure time by to decide how many
         snaps to use. Default 60s.
+    event_gen_detailers : `list`
+        A list of detailers to run on arrays right after generating
+        a list.
+    simple_single_tesselate : `bool`
+        If tesselating a single HEALpixel, use the center of the
+        HEALpixel for the pointing rather than try to look up
+        pointings that cover the center and all corners. Default True.
+    dither_per_visit : `bool`
+        If nvis > 1, randomize the tesselation between each visit.
+        Probably what you want to do if you're searching for an event
+        and worried about chip and raft gaps. Default True.
     """
 
     def __init__(
@@ -74,13 +94,15 @@ class ToOScriptedSurvey(ScriptedSurvey, BaseMarkovSurvey):
         followup_footprint=None,
         nside=DEFAULT_NSIDE,
         reward_val=1e6,
-        times=[1, 2, 4, 24, 48],
+        times=[1, 2, 4, 8, 24, 48],
         bands_at_times=["gz", "gz", "gz", "gz", "gz", "gz"],
         nvis=[1, 1, 1, 1, 6, 6],
         exptimes=[DEFAULT_EXP_TIME] * 6,
         camera="LSST",
         survey_name="ToO",
         target_name_base="ToO",
+        observation_reason="ToO",
+        science_program=None,
         flushtime=2.0,
         mjd_tol=1.0 / 24.0,
         dist_tol=0.5,
@@ -101,6 +123,10 @@ class ToOScriptedSurvey(ScriptedSurvey, BaseMarkovSurvey):
         split_long_max=30.0,
         split_long_div=60.0,
         filters_at_times=None,
+        event_gen_detailers=None,
+        return_n_limit=500,
+        simple_single_tesselate=True,
+        dither_per_visit=True,
     ):
         if filters_at_times is not None:
             warnings.warn("filters_at_times deprecated in favor of bands_at_times", FutureWarning)
@@ -116,6 +142,7 @@ class ToOScriptedSurvey(ScriptedSurvey, BaseMarkovSurvey):
             ignore_obs = []
 
         self.basis_functions = basis_functions
+        self.basis_weights = [0] * len(basis_functions)
         self.survey_name = survey_name
         self.followup_footprint = followup_footprint
         self.last_event_id = -1
@@ -136,12 +163,21 @@ class ToOScriptedSurvey(ScriptedSurvey, BaseMarkovSurvey):
         self.HA_min = HA_min
         self.HA_max = HA_max
         self.ignore_obs = ignore_obs
+        self.return_n_limit = return_n_limit
         self.extra_features = {}
         self.extra_basis_functions = {}
-        self.detailers = []
+        self.simple_single_tesselate = simple_single_tesselate
+        self.dither_per_visit = dither_per_visit
+        if detailers is None:
+            self.detailers = []
+        else:
+            self.detailers = detailers
+        if event_gen_detailers is None:
+            self.event_gen_detailers = []
+        else:
+            self.event_gen_detailers = event_gen_detailers
         self.dither = dither
         self.id_start = id_start
-        self.detailers = detailers
         self.last_mjd = -1
         self.too_types_to_follow = too_types_to_follow
         self.split_long = split_long
@@ -162,8 +198,8 @@ class ToOScriptedSurvey(ScriptedSurvey, BaseMarkovSurvey):
             ValueError('camera %s unknown, should be "LSST" or "comcam"' % camera)
         self.fields = self.fields_init.copy()
 
-        self.hp2fields = np.array([])
         self._hp2fieldsetup(self.fields["RA"], self.fields["dec"])
+        self._hpcorners2fieldsetup(self.fields["RA"], self.fields["dec"])
 
         # Don't bother with checking if we can run before twilight ends
         self.before_twi_check = False
@@ -184,30 +220,30 @@ class ToOScriptedSurvey(ScriptedSurvey, BaseMarkovSurvey):
 
         # list to keep track of alerts we have already seen
         self.seen_alerts = []
-
-    def _check_list(self, conditions):
-        """Check to see if the current mjd is good"""
-        observation = None
-        if self.obs_wanted.size > 0:
-            # Scheduled observations that are in the right
-            # time window and have not been executed
-            in_time_window = np.where(
-                (self.mjd_start < conditions.mjd)
-                & (self.obs_wanted["flush_by_mjd"] > conditions.mjd)
-                & (~self.obs_wanted["observed"])
-            )[0]
-
-            if np.size(in_time_window) > 0:
-                pass_checks = self._check_alts_ha(self.obs_wanted[in_time_window], conditions)
-                matches = in_time_window[pass_checks]
+        # Check if there is a detailer for tracking info
+        has_tracking_detailer = False
+        for detailer in self.detailers:
+            if isinstance(detailer, TrackingInfoDetailer):
+                has_tracking_detailer = True
+        # Add information for visit metadata if necessary
+        if (science_program is not None) | (observation_reason is not None):
+            should_have_tracking_detailer = True
+        else:
+            should_have_tracking_detailer = False
+        if should_have_tracking_detailer:
+            # Check if one already present - will use that if so.
+            if has_tracking_detailer:
+                warnings.warn(
+                    f"Survey {self.survey_name} has a tracking detailer but "
+                    f"observation_reason or science_program also set (and will ignore)."
+                )
             else:
-                matches = []
-
-            if np.size(matches) > 0:
-                observation = ScheduledObservationArray(n=1)
-                observation[0] = self.obs_wanted[matches[0]]
-
-        return observation
+                self.detailers.append(
+                    TrackingInfoDetailer(
+                        science_program=science_program,
+                        observation_reason=observation_reason,
+                    )
+                )
 
     def flush_script(self, conditions):
         """Remove things from the script that aren't needed anymore"""
@@ -256,17 +292,44 @@ class ToOScriptedSurvey(ScriptedSurvey, BaseMarkovSurvey):
         # XXX-may be doing some ra,dec to conversions xyz more
         # than needed.
         self._hp2fieldsetup(ra, dec)
+        self._hpcorners2fieldsetup(ra, dec)
 
         # Advance the spin index
         self.spin_indx += 1
 
+    def _hpcorners2fieldsetup(self, ra, dec):
+        """Map each healpixel corner to nearest field.
+        Parameters
+        ----------
+        ra : `float`
+            The RA of the possible pointings (radians)
+        dec : `float`
+            The decs of the possible pointings (radians)
+        """
+        hpid = np.arange(hp.nside2npix(self.nside))
+        xyz = hp.boundaries(self.nside, hpid)
+        tree = _build_tree(ra, dec)
+
+        self.hpcorners2fields = []
+        for i in range(4):
+            dist, ind = tree.query(np.vstack([xyz[:, 0, i], xyz[:, 1, i], xyz[:, 2, i]]).T, k=1)
+            self.hpcorners2fields.append(ind)
+
     def _tesselate(self, hpid_to_observe):
-        self._spin_fields()
-        field_ids = np.unique(self.hp2fields[hpid_to_observe])
-        # Put the fields in a good order.
-        better_order = order_observations(self.fields["RA"][field_ids], self.fields["dec"][field_ids])
-        ras = self.fields["RA"][field_ids[better_order]]
-        decs = self.fields["dec"][field_ids[better_order]]
+
+        if self.simple_single_tesselate & (len(hpid_to_observe) == 1):
+            ras, decs = _hpid2_ra_dec(self.nside, hpid_to_observe)
+        else:
+            self._spin_fields()
+            # Closest pointings to all the HEALpix centers
+            center_field_ids = np.unique(self.hp2fields[hpid_to_observe])
+            # Closest pointings to all the HEALpix corners
+            corners = np.concatenate([np.unique(ind[hpid_to_observe]) for ind in self.hpcorners2fields])
+            field_ids = np.unique(np.concatenate([center_field_ids, corners]))
+            # Put the fields in a good order
+            better_order = order_observations(self.fields["RA"][field_ids], self.fields["dec"][field_ids])
+            ras = self.fields["RA"][field_ids[better_order]]
+            decs = self.fields["dec"][field_ids[better_order]]
 
         return ras, decs
 
@@ -292,7 +355,11 @@ class ToOScriptedSurvey(ScriptedSurvey, BaseMarkovSurvey):
             # we want to observe
             hpid_center = _ra_dec2_hpid(self.nside, target_o_o.ra_rad_center, target_o_o.dec_rad_center)
             if self.followup_footprint[hpid_center] > 0:
-                target_area = self.followup_footprint * target_o_o.footprint
+                # ToO footprint could be any nside, so match here
+                # hp.ud_grade takes a mean. So a 1/0 mask will be forgiving
+                # and expand, but a mask using np.nan will be hard.
+                matched_target_o_o_fp = hp.ud_grade(target_o_o.footprint, self.nside)
+                target_area = self.followup_footprint * matched_target_o_o_fp
                 # generate a list of pointings for that area
                 hpid_to_observe = np.where(target_area > 0)[0]
                 if hpid_to_observe.size > 0:
@@ -327,9 +394,11 @@ class ToOScriptedSurvey(ScriptedSurvey, BaseMarkovSurvey):
                     self.exptimes,
                     np.arange(np.size(self.times)),
                 ):
+                    # Could potentially throw in a dither change here
+                    # so it is different at each time as well?
                     for i in range(nv):
                         # let's dither each pointing
-                        if (i != 0) & (hpid_to_observe.size > 0):
+                        if (i != 0) & (hpid_to_observe.size > 0) & (self.dither_per_visit):
                             ras, decs = self._tesselate(hpid_to_observe)
 
                         for bandname in bandnames:
@@ -378,19 +447,24 @@ class ToOScriptedSurvey(ScriptedSurvey, BaseMarkovSurvey):
                             obs["target_name"] = self.target_name_base + "_i%i" % index
                             obs_list.append(obs)
                 observations = np.concatenate(obs_list)
+                for detailer in self.event_gen_detailers:
+                    observations = detailer(observations, conditions, target_o_o=target_o_o)
                 self.set_script(observations)
 
-    def calc_reward_function(self, conditions):
-        """If there is an observation ready to go, execute it,
-        otherwise, -inf"""
-        # check if any new event has come in
-
+    def update_conditions(self, conditions):
         if conditions.targets_of_opportunity is not None:
             for target_o_o in conditions.targets_of_opportunity:
                 if target_o_o.id > self.last_event_id:
                     self._new_event(target_o_o, conditions)
                     self.last_event_id = target_o_o.id
 
+    def calc_reward_function(self, conditions):
+        """If there is an observation ready to go, execute it,
+        otherwise, -inf"""
+
+        # check if any new event has come in
+        self.update_conditions(conditions)
+
         observation = self.generate_observations_rough(conditions)
 
         if (observation is None) | (len(observation) == 0):
@@ -430,9 +504,14 @@ def gen_too_surveys(
     split_long=False,
     long_exp_nsnaps=2,
     n_snaps=2,
+    wind_speed_maximum=20.0,
+    observation_reason="ToO",
+    science_program=None,
 ):
     result = []
-
+    bf_list = []
+    bf_list.append(basis_functions.AvoidDirectWind(wind_speed_maximum=wind_speed_maximum, nside=nside))
+    bf_list.append(basis_functions.MoonAvoidanceBasisFunction(moon_distance=30.0))
     ############
     # Generic GW followup
     ############
@@ -450,63 +529,48 @@ def gen_too_surveys(
     exptimes = [120.0, 120.0, 120.0]
     result.append(
         ToOScriptedSurvey(
-            [],
+            bf_list,
             nside=nside,
             followup_footprint=too_footprint,
             times=times,
             bands_at_times=bands_at_times,
             nvis=nvis,
             exptimes=exptimes,
-            detailers=detailer_list,
+            detailers=deepcopy(detailer_list),
             too_types_to_follow=["GW_case_A"],
             survey_name="ToO, GW_case_A",
             split_long=split_long,
             flushtime=48.0,
             n_snaps=long_exp_nsnaps,
             target_name_base="GW_case_A",
+            observation_reason=observation_reason,
+            science_program=science_program,
         )
     )
 
-    times = [0, 24, 48, 72]
-    bands_at_times = ["gri", "ri", "ri", "ri"]
-    nvis = [3, 1, 1, 1]
-    exptimes = [120.0, 180.0, 180.0, 180.0]
+    ############
+    # GW gold and GW unidentified gold
+    ############
+
+    times = [0, 2, 4, 24, 48, 72]
+    bands_at_times = ["gri", "gri", "gri", "ri", "ri", "ri"]
+    nvis = [4, 4, 4, 6, 6, 6]
+    exptimes = [30.0, 30.0, 30.0, 30.0, 30.0, 30.0]
     result.append(
         ToOScriptedSurvey(
-            [],
+            bf_list,
             nside=nside,
             followup_footprint=too_footprint,
             times=times,
             bands_at_times=bands_at_times,
             nvis=nvis,
             exptimes=exptimes,
-            detailers=detailer_list,
+            detailers=deepcopy(detailer_list),
             too_types_to_follow=["GW_case_B", "GW_case_C"],
             survey_name="ToO, GW_case_B_C",
             target_name_base="GW_case_B_C",
-            split_long=split_long,
-            flushtime=48,
-            n_snaps=long_exp_nsnaps,
-        )
-    )
-
-    times = [0, 24, 48, 72]
-    bands_at_times = ["gr", "gr", "gr", "gr"]
-    nvis = [1, 1, 1, 1]
-    exptimes = [120.0, 120.0, 120.0, 120.0]
-    result.append(
-        ToOScriptedSurvey(
-            [],
-            nside=nside,
-            followup_footprint=too_footprint,
-            times=times,
-            bands_at_times=bands_at_times,
-            nvis=nvis,
-            exptimes=exptimes,
-            detailers=detailer_list,
-            too_types_to_follow=["GW_case_D", "GW_case_E"],
-            survey_name="ToO, GW_case_D_E",
-            target_name_base="GW_case_D_E",
+            observation_reason=observation_reason,
+            science_program=science_program,
             split_long=split_long,
             flushtime=48,
             n_snaps=long_exp_nsnaps,
@@ -514,32 +578,80 @@ def gen_too_surveys(
     )
 
     ############
-    # Black hole-black hole GW merger
+    # GW silver and GW unidentified silver
+    ############
+
+    times = [0, 24, 48, 72]
+    bands_at_times = ["gi", "gi", "gi", "gi"]
+    nvis = [1, 4, 4, 4]
+    exptimes = [30.0, 30.0, 30.0, 30.0]
+    result.append(
+        ToOScriptedSurvey(
+            bf_list,
+            nside=nside,
+            followup_footprint=too_footprint,
+            times=times,
+            bands_at_times=bands_at_times,
+            nvis=nvis,
+            exptimes=exptimes,
+            detailers=deepcopy(detailer_list),
+            too_types_to_follow=["GW_case_D", "GW_case_E"],
+            survey_name="ToO, GW_case_D_E",
+            target_name_base="GW_case_D_E",
+            observation_reason=observation_reason,
+            science_program=science_program,
+            split_long=split_long,
+            flushtime=48,
+            n_snaps=long_exp_nsnaps,
+            event_gen_detailers=None,
+        )
+    )
+
+    ############
+    # BBH hole-black hole GW merger
+    # If nearby (dist < 2200 Mpc) and dark time, use ugi
+    # If distant (dist > 2200 Mpc) and dark time, use rgi
+    # If bright time, use rzi
     ############
 
     # XXX--only considering bright objects now.
 
+    # SM-- adding support for different BBH cases. We should have
+    # a discussion about how to differentiate these, and if it is
+    # possible to do so through the alert stream.
+
+    event_detailers = [
+        BandPickToODetailer(
+            band_start="z", band_end="g", distance_limit=30e10, check_mounted=True, require_dark=True
+        ),
+        BandPickToODetailer(
+            band_start="r", band_end="u", distance_limit=2.2e6, check_mounted=True, require_dark=True
+        ),
+    ]
     times = np.array([0, 2, 7, 9, 39]) * 24
-    bands_at_times = ["ugri"] * 5
-    nvis = [1] * 5
-    exptimes = [DEFAULT_EXP_TIME] * 5
+    bands_at_times = ["rzi"] * times.size
+    nvis = [1] * times.size
+    exptimes = [DEFAULT_EXP_TIME] * times.size
 
     result.append(
         ToOScriptedSurvey(
-            [],
+            bf_list,
             nside=nside,
             followup_footprint=too_footprint,
             times=times,
             bands_at_times=bands_at_times,
             nvis=nvis,
             exptimes=exptimes,
-            detailers=detailer_list,
+            detailers=deepcopy(detailer_list),
             too_types_to_follow=["BBH_case_A", "BBH_case_B", "BBH_case_C"],
             survey_name="ToO, BBH",
             target_name_base="BBH",
+            observation_reason=observation_reason,
+            science_program=science_program,
             split_long=split_long,
             flushtime=48,
             n_snaps=n_snaps,
+            event_gen_detailers=event_detailers,
         )
     )
 
@@ -547,50 +659,53 @@ def gen_too_surveys(
     # Lensed BNS
     ############
 
-    times = [1.0, 1.0]
-    bands_at_times = ["g", "r"]
-    nvis = [1, 1]
-    exptimes = [DEFAULT_EXP_TIME, 90.0]
+    times = np.array([1.0, 1.0, 25, 25, 49, 49])
+    bands_at_times = ["g", "r"] * 3
+    nvis = [1, 3] * 3
+    exptimes = [DEFAULT_EXP_TIME, DEFAULT_EXP_TIME] * 3
 
     result.append(
         ToOScriptedSurvey(
-            [],
+            bf_list,
             nside=nside,
             followup_footprint=too_footprint,
             times=times,
             bands_at_times=bands_at_times,
             nvis=nvis,
             exptimes=exptimes,
-            detailers=detailer_list,
+            detailers=deepcopy(detailer_list),
             too_types_to_follow=["lensed_BNS_case_A"],
             survey_name="ToO, LensedBNS_A",
             target_name_base="LensedBNS_A",
+            observation_reason=observation_reason,
+            science_program=science_program,
             split_long=split_long,
             flushtime=48.0,
             n_snaps=n_snaps,
         )
     )
 
-    times = [1.0]
-    bands_at_times = [
-        "gr",
-    ]
-    nvis = [10]
-    exptimes = [150]
+    # This is the small skymap (15 deg^2 case)
+    times = np.array([1.0, 1.0, 25, 25, 49, 49])
+    bands_at_times = ["g", "r"] * 3
+    nvis = [180, 120] * 3
+    exptimes = [30] * times.size
 
     result.append(
         ToOScriptedSurvey(
-            [],
+            bf_list,
             nside=nside,
             followup_footprint=too_footprint,
             times=times,
             bands_at_times=bands_at_times,
             nvis=nvis,
             exptimes=exptimes,
-            detailers=detailer_list,
+            detailers=deepcopy(detailer_list),
             too_types_to_follow=["lensed_BNS_case_B"],
             survey_name="ToO, LensedBNS_B",
             target_name_base="LensedBNS_B",
+            observation_reason=observation_reason,
+            science_program=science_program,
             split_long=split_long,
             flushtime=48.0,
             n_snaps=long_exp_nsnaps,
@@ -601,62 +716,37 @@ def gen_too_surveys(
     # Neutrino detector followup
     ############
 
-    # XXX--need to update footprint to cut out galactic latitude
-
-    times = [0, 0, 15 / 60.0, 0.5, 24, 24.5, 144]
-    bands_at_times = ["g", "r", "z", "g", "r", "z", "grz"]
+    times = [0, 0, 15 / 60.0, 0, 24, 24, 144, 144]
+    bands_at_times = ["u", "g", "r", "z", "g", "r", "g", "rz"]
     exptimes = [
-        120,
+        30,
+        30,
         DEFAULT_EXP_TIME,
         DEFAULT_EXP_TIME,
-        120,
-        DEFAULT_EXP_TIME,
+        30,
         DEFAULT_EXP_TIME,
+        30,
         DEFAULT_EXP_TIME,
     ]
-    nvis = [1, 1, 1, 1, 1, 1, 1]
+    nvis = [1, 4, 1, 1, 4, 1, 4, 1]
 
     result.append(
         ToOScriptedSurvey(
-            [],
+            bf_list,
             nside=nside,
             followup_footprint=too_footprint,
             times=times,
             bands_at_times=bands_at_times,
             nvis=nvis,
             exptimes=exptimes,
-            detailers=detailer_list,
+            detailers=deepcopy(detailer_list),
             too_types_to_follow=["neutrino"],
             survey_name="ToO, neutrino",
             target_name_base="neutrino",
+            observation_reason=observation_reason,
+            science_program=science_program,
             split_long=split_long,
-            flushtime=8.0,
-            n_snaps=n_snaps,
-        )
-    )
-
-    times = [0]
-    bands_at_times = ["u"]
-    exptimes = [DEFAULT_EXP_TIME]
-    nvis = [1]
-
-    # U-band with very long flush time.
-
-    result.append(
-        ToOScriptedSurvey(
-            [],
-            nside=nside,
-            followup_footprint=too_footprint,
-            times=times,
-            bands_at_times=bands_at_times,
-            nvis=nvis,
-            exptimes=exptimes,
-            detailers=detailer_list,
-            too_types_to_follow=["neutrino"],
-            survey_name="ToO, neutrino_u",
-            target_name_base="neutrino_u",
-            split_long=split_long,
-            flushtime=1440,
+            flushtime=20 * 24,
             n_snaps=n_snaps,
         )
     )
@@ -675,19 +765,21 @@ def gen_too_surveys(
 
     result.append(
         ToOScriptedSurvey(
-            [],
+            bf_list,
             nside=nside,
             followup_footprint=too_footprint,
             times=times,
             bands_at_times=bands_at_times,
             nvis=nvis,
             exptimes=exptimes,
-            detailers=detailer_list,
+            detailers=deepcopy(detailer_list),
             too_types_to_follow=["SSO_night"],
             survey_name="ToO, SSO_night",
             target_name_base="SSO_night",
+            observation_reason=observation_reason,
+            science_program=science_program,
             split_long=split_long,
-            flushtime=2.0,
+            flushtime=3.0,
             n_snaps=n_snaps,
         )
     )
@@ -699,19 +791,54 @@ def gen_too_surveys(
 
     result.append(
         ToOScriptedSurvey(
-            [],
+            bf_list,
             nside=nside,
             followup_footprint=too_footprint,
             times=times,
             bands_at_times=bands_at_times,
             nvis=nvis,
             exptimes=exptimes,
-            detailers=detailer_list,
+            detailers=deepcopy(detailer_list),
             too_types_to_follow=["SSO_twilight"],
             survey_name="ToO, SSO_twi",
             target_name_base="SSO_twi",
+            observation_reason=observation_reason,
+            science_program=science_program,
             split_long=split_long,
-            flushtime=2.0,
+            flushtime=3.0,
+            n_snaps=n_snaps,
+        )
+    )
+
+    ############
+    # Galactic Supernova
+    ############
+    # For galactic supernovae, we want to tile continuously
+    # in the region in 1s and 15s exposures for i band, until
+    # a counterpart is identified
+
+    times = [0, 0, 0, 0] * 4
+    bands_at_times = ["i", "i", "i", "i"] * 4
+    nvis = [1, 1, 1, 1] * 4
+    exptimes = [1, 15, 1, 15] * 4
+
+    result.append(
+        ToOScriptedSurvey(
+            bf_list,
+            nside=nside,
+            followup_footprint=too_footprint,
+            times=times,
+            bands_at_times=bands_at_times,
+            nvis=nvis,
+            exptimes=exptimes,
+            detailers=deepcopy(detailer_list),
+            too_types_to_follow=["SN_Galactic"],
+            survey_name="ToO, galactic SN",
+            target_name_base="SN_Galactic",
+            observation_reason=observation_reason,
+            science_program=science_program,
+            split_long=split_long,
+            flushtime=48.0,
             n_snaps=n_snaps,
         )
     )
