@@ -16,6 +16,7 @@ __all__ = (
     "slice_wfd_indx",
     "slice_quad_galactic_cut",
     "make_rolling_footprints",
+    "PerFilterStep",
 )
 
 import warnings
@@ -589,3 +590,75 @@ def calc_norm_factor_array(goal_map, radius=1.75):
     hp_area = hp.nside2pixarea(nside, degrees=True)
     norm_val = radius**2 * np.pi / hp_area / all_maps_sum
     return norm_val
+
+
+class PerFilterStep(BasePixelEvolution):
+    """Make a custom step function per filter
+    Compensate for filters not always being loaded.
+    This does not use season information, so only
+    useful for a survey of limited duration/limited
+    RA range (e.g., a science validation survey).
+
+    Parameters
+    ----------
+    survey_length : `float`
+      Suervey length in days. Default 80.
+    bands : `list`
+        Bandnames. Defualt of none uses ugrizy
+    loaded_dict : `dict`
+        Dict with keys of bandnames and values of
+        arrays of ints with the nights that band is
+        loaded. E.g.,
+        loaded_dict = {'u': np.array([1,2,3,15,16,17])}
+        means the u band will only be availble on those
+        nights.
+    """
+
+    def __init__(
+        self,
+        survey_length=80,
+        bands=None,
+        loaded_dict=None,
+    ):
+
+        if bands is None:
+            bands = ["u", "g", "r", "i", "z", "y"]
+        if loaded_dict is None:
+            loaded_dict = {}
+
+        self.bands = bands
+        self.survey_length = survey_length
+
+        self.bands2indx = {}
+        for i, bandname in enumerate(bands):
+            self.bands2indx[bandname] = i
+
+        self.slopes = {}
+        self.loaded_dict = {}
+
+        for bandname in bands:
+            if bandname in loaded_dict.keys():
+                self.loaded_dict[bandname] = loaded_dict[bandname]
+                self.slopes[bandname] = 1.0 / self.loaded_dict[bandname].size
+
+    def __call__(self, t_elapsed, phase):
+        """
+        Parameters
+        ----------
+        t_elapsed : `float`
+            Time elapsed in the survey (days).
+        """
+
+        # filters all the time evolve linearly increase between
+        # 0 and 1 for length of survey.
+        frac_done = t_elapsed / self.survey_length
+
+        # broadcast out to n_filters
+        result = np.tile(phase * 0 + frac_done, (len(self.bands), 1))
+        for bandname in self.loaded_dict:
+            days_completed = np.where(self.loaded_dict[bandname] <= t_elapsed)[0].size
+            result[self.bands2indx[bandname], :] = (
+                result[self.bands2indx[bandname], :] * 0 + days_completed * self.slopes[bandname]
+            )
+
+        return result
