@@ -2,8 +2,10 @@ __all__ = ("DEFAULT_NSIDE", "SURVEY_START_MJD", "survey_start_mjd")
 
 import warnings
 
+from astropy.time import Time
+
 DEFAULT_NSIDE = 32  # HEALpix nside, ~1.83 degree resolution
-SURVEY_START_MJD = 60980.0  # 60980 = Nov 1, 2025
+SURVEY_START_MJD = Time("2025-11-01T12:00:00").mjd
 
 
 def survey_start_mjd():
